/*
 * This file is part of the OWL API.
 * 
 * The contents of this file are subject to the LGPL License, Version 3.0.
 * 
 * Copyright (C) 2011, The University of Queensland
 * 
 * This program is free software: you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program. If
 * not, see http://www.gnu.org/licenses/.
 * 
 * 
 * Alternatively, the contents of this file may be used under the terms of the Apache License,
 * Version 2.0 in which case, the provisions of the Apache License Version 2.0 are applicable
 * instead of those above.
 * 
 * Copyright 2011, The University of Queensland
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
package org.semanticweb.owlapi.rio.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

import org.openrdf.model.Resource;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.model.impl.ValueFactoryImpl;
import org.semanticweb.owlapi.io.RDFLiteral;
import org.semanticweb.owlapi.io.RDFResourceIRI;
import org.semanticweb.owlapi.io.RDFTriple;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Peter Ansell p_ansell@yahoo.com
 * 
 */
public class RioUtils
{
    private static final Logger log = LoggerFactory.getLogger(RioUtils.class);
    
    /**
     * Create a Statement with an empty context.
     * 
     * @param triple
     * @return
     */
    public static Statement tripleAsStatement(final RDFTriple triple)
    {
        Collection<Statement> statements = RioUtils.tripleAsStatements(triple);
        
        if(!statements.isEmpty())
        {
            return statements.iterator().next();
        }
        else
        {
            return null;
        }
    }
    
    /**
     * Create a Statement with the given context.
     * 
     * @param triple
     * @param context
     *            If context is not null, it is used to create a context statement
     * @return
     */
    public static Collection<Statement> tripleAsStatements(final RDFTriple triple, final Resource... contexts)
    {
        final ValueFactoryImpl vf = ValueFactoryImpl.getInstance();
        
        Resource subject;
        URI predicate;
        Value object;
        
        if(triple.getSubject() instanceof RDFResourceIRI)
        {
            try
            {
                subject = vf.createURI(triple.getSubject().getIRI().toString());
            }
            catch(IllegalArgumentException iae)
            {
                log.error("Subject URI was invalid: {}", triple);
                return Collections.emptyList();
            }
        }
        else
        {
            // FIXME: When blank nodes are no longer represented as IRIs internally, need to fix
            // this
            if(triple.getSubject().getIRI().toString().startsWith("_:"))
            {
                subject = vf.createBNode(triple.getSubject().getIRI().toString().substring(2));
            }
            else
            {
                subject = vf.createBNode(triple.getSubject().getIRI().toString());
            }
        }
        
        predicate = vf.createURI(triple.getPredicate().getIRI().toString());
        
        if(triple.getObject() instanceof RDFResourceIRI)
        {
            try
            {
                object = vf.createURI(triple.getObject().getIRI().toString());
            }
            catch(IllegalArgumentException iae)
            {
                log.error("Object URI was invalid: {}", triple);
                return Collections.emptyList();
            }
        }
        else if(triple.getObject() instanceof RDFLiteral)
        {
            final RDFLiteral literalObject = (RDFLiteral)triple.getObject();
            if(literalObject.hasDatatype())
            {
                object =
                        vf.createLiteral(literalObject.stringValue(),
                                vf.createURI(literalObject.getDatatype().toString()));
            }
            else if(literalObject.hasLang())
            {
                object = vf.createLiteral(literalObject.stringValue(), literalObject.getLang());
            }
            else
            {
                object = vf.createLiteral(literalObject.stringValue());
            }
        }
        else
        {
            // FIXME: When blank nodes are no longer represented as IRIs internally, need to fix
            // this
            if(triple.getObject().getIRI().toString().startsWith("_:"))
            {
                object = vf.createBNode(triple.getObject().getIRI().toString().substring(2));
            }
            else
            {
                object = vf.createBNode(triple.getObject().getIRI().toString());
            }
        }
        
        if(contexts == null || contexts.length == 0)
        {
            return Collections.singletonList(vf.createStatement(subject, predicate, object));
        }
        else
        {
            final ArrayList<Statement> results = new ArrayList<Statement>(contexts.length);
            
            for(final Resource nextContext : contexts)
            {
                results.add(vf.createStatement(subject, predicate, object, nextContext));
            }
            
            return results;
        }
        
    }
    
}
